% Ce code est un exemple permettant de réaliser le krigeage ordinaire pour
% le jeu de données de WalkerLake avec 5000 données. 

% Livrable 
% 1 - Étudier l'impact du nombre de données sur la variance de krigeage
%     Utilisez parmis les 14 fichiers disponibles 
% 2 - Étudier l'impact du voisinage sur les estimations du krigeage ordinaire.
%     Le choix de la taille du voisinnage est libre de 2 à 500, p.ex. 

%% Créer un modèle synthétique 3D :
% data contient les données [X, Y, Z, valeur ]
model = [1 0 0 0 0 0 0; 4 20 10 10 0 0 0]; c= [0.2; 0.8]; % pépite + sphérique isotrope
nx = 50; dx =1; ny=50; dy =1; nz=50; dz=1;
seed = 152452; % Pour la reproductibilité
nbsimul =1; % Une référence
[datasim]=fftma(model,c,seed,nbsimul,nx,dx,ny,dy,nz,dz);

%% On échantillonne 1000 données 
nbdata = 1000;
% Générer une séquence de Halton pour la 3D
p = haltonset(3);

% Points d'échantillonnage de la séquence de Halton
id_x = floor(p(100+(nbdata +1):100+(nbdata +1)+nbdata-1,1)*nx) + 1;
id_y = floor(p(100+(nbdata +1):100+(nbdata +1)+nbdata-1,2)*ny) + 1;
id_z = floor(p(100+(nbdata +1):100+(nbdata +1)+nbdata-1,3)*nz) + 1;

% Combiner les coordonnées
id = sortrows([id_x id_y id_z], [1 2 3]);

% Générer des indices 3D uniques
idx = unique((id(:,1)-1)*ny*nz + (id(:,2)-1)*nz + id(:,3));

data = datasim(idx,:);
%% Calcul des statistiques descriptives avec statelem.
% À faire.
% Moyene, écart-type, histogramme, etc.


%% Calcul des variogrammes expérimentaux.
% Le programme utilise varioexp2d.m pour calculer les variogrammes expérimentaux.
% Le programme utilise ajustermulti.m pour ajuster un modèle anisotrope théorique.
% Suivez attentivement l'exemple pour l'application.

% Paramètres pour les variogrammes expérimentaux
nbclas = 10;                         % Nombre de classes pour h
lclas=[(0:nbclas-1);(1:nbclas)]'*4;  % Discrétisation des classes
vdir=[ [0,1,2,3,4,5,6,7,0]'*22.5 , [0,0,0,0,0,0,0,0,90]' ];      % Directions d'anisotropie    [Azimut , Plongé]
vreg=[ [10,10,10,10,10,10,10,10,10]' , [10,10,10,10,10,10,10,10,10]' ]; % Tolérance angulaire  [Azimut , Plongé]
 
figure(1)
% Calcul des variogrammes expérimentaux
[gexp] = varioexp3d(data, nbclas, lclas, vdir, vreg);

% Paramètres pour l'ajustement des variogrammes expérimentaux
% Ajustement pondéré en fonction du nombre de paires de points.
expoh = 2;                            % Exposant pour pondérer l'importance
hmin = 1;                             % Classe h=0 pour gérer le bruit
ccode = [0; 0];                       % (0 libre et 1 imposé)
modelcode=[1 1 1 1 1 1 1  ;  1 0 0 0 0 0 0]; % Imposition de l effet de pépite et modèle exponentiel
varExp = var(data(:,4));
c = [0.1 * varExp; 0.9 * varExp];     % Ajuster si nécessaire
model=[1 1 1 1 1 1 1  ; 4 20 20 30 0 0 0]; % Ajuster les paramètres selon les besoins

% Ajustement des variogrammes expérimentaux
figure(100)
[model, c, ~] = ajustermulti(gexp, model, c, modelcode, ccode, vdir, vreg, expoh, hmin);

%% Réaliser le krigeage ordinaire avec les modèles ajustés
% Paramètres pour la grille de krigeage
xmin = 1; xmax = nx; dx = 1;
ymin = 1; ymax = ny; dy = 1;
zmin = 1; zmax = ny; dz = 1;
x0 = grille3(xmin, xmax, dx, ymin, ymax, dy, zmin, zmax, dz);

% Paramètres de krigeage
itype = 3;                % Krigeage ordinaire
avg = 0;                  % Moyenne des données (non utilisée en krigeage ordinaire)
block = [1 1 1];          % Taille du bloc de krigeage
nd = [1 1 1];             % Discrétisation du bloc
ival = 0;                 % Pas de validation croisée
nk = 16;                  % Nombre de données dans le voisinage (modifier pour étudier l'impact)
r = 9999;                 % Rayon de recherche circulaire maximal
ntok = 10;                % Nombre de points krigés simultanément

[krig, skrig, svbloc] = cokri(data, x0, model, c, itype, avg, block, nd, ival, nk, r, ntok);

%% Visualisation des résultats du krigeage ordinaire
% Définir la taille de la grille
nx = (xmax - xmin) / dx + 1;
ny = (ymax - ymin) / dy + 1;
nz = (zmax - zmin) / dz + 1;

% Création des coordonnées du maillage
[x_grid, y_grid, z_grid] = meshgrid(linspace(xmin, xmax, nx), ...
                                     linspace(ymin, ymax, ny), ...
                                     linspace(zmin, zmax, nz));

% Reshape des résultats du krigeage
t = real(reshape(krig(:,4,1), nx, ny, nz));  % Valeurs krigées reshaped en 3D
std_krig = reshape(real(sqrt(skrig(:,4,1))), nx, ny, nz);  % Écart-type du krigeage reshaped en 3D

% Visualisation des résultats du krigeage en 3D

% Visualisation des résultats du krigeage - Concentration
figure(33); clf
slice(x_grid, y_grid, z_grid, t, [], [], linspace(zmin, zmax, nz/2)); % Tranches pour la vue 3D
title('Carte krigée');
xlabel('X'); ylabel('Y'); zlabel('Z');
colorbar;
clim([-3 3]); % Ajuster l'échelle de couleur
set(gca, 'fontsize', 8);

% Visualisation de l'écart-type du krigeage
figure(34); clf
slice(x_grid, y_grid, z_grid, std_krig, [], [], linspace(zmin, zmax, nz/2)); % Tranches pour l'erreur en 3D
hold on;
scatter3(data(:,1), data(:,2), data(:,3), 1, 'w');  % Points de données en 3D avec taille ajustée
hold off;
axis equal;
xlabel('X'); ylabel('Y'); zlabel('Z');
colorbar;
clim([0 1]); % Ajuster l'échelle de couleur en fonction des données
set(gca, 'fontsize', 8);
title('Écart-type du krigeage');



