function [gexp]=varioexp3d(x,nbclas,lclas,vdir,vreg)
%
% syntaxe function [gexp]=varioexp3d(x,nbclas,lclas,vdir,vreg)
%
%
% input   x : matrice n*4 contenant les coordonnees (x,y,z) et la variable;
%         nbclas: nombre de classes dans les calculs de vario.
%         lclas: largeur des classes (scalaire)
%                ou matrice(nbclas x 2) donnant les limites des classes (>binf, <=bsup)
%         vdir: matrice ndirx2. Chaque range contient direction (azimut) et pendage (positif vers le bas) 
%         vreg: regularisation (degre 0-90) angle de part et d'autre de la direction considre doit etre de meme taille que vdir
%
% output  gexp: matrice (3d) contenant les distances moyennes (1 ere colonne)
%                                 le nombre de paires    (2e colonne)
%                                 le variogramme exprimental(3e colonne)
%        1 plan par direction demande et dans l'ordre demand
%
% auteur D. Marcotte, avril. 2002


if length(lclas)==1
   lclas=[[0:lclas:(nbclas-1)*lclas]',[lclas:lclas:(nbclas)*lclas]'];
end
ncl=length(lclas);

if size(vdir,1)~=length(vreg)
   display('Fournir autant de directions que de rgularisations')
   return
end
ndir=size(vdir,1);

n=length(x);
k=0;
var=[];
[n,p]=size(x);
xx=x(:,1);
yy=x(:,2);
zz=x(:,3);
v=x(:,4);
gexp=zeros(ncl,3,ndir);
nplot=ceil(sqrt(ndir));
u=poletocart(vdir); % u contient les directions pendages sous forme de vecteurs unitaires (cosinus directeurs)
tol=cosd(vreg);

% former les indices de paires


for i=1:n-1,  % boucle des observations
   ind=[ones((n-i),1)*i,[(i+1):n]'];
   yt=ind;                     % vecteur des indices  
   
   % Calcul des distances 
   
   ht=sqrt((xx(yt(:,1))-xx(yt(:,2))).^2+(yy(yt(:,1))-yy(yt(:,2))).^2+(zz(yt(:,1))-zz(yt(:,2))).^2);
   
   
   % calcul des angles
   dx=xx(yt(:,2))-xx(yt(:,1));
   dy=yy(yt(:,2))-yy(yt(:,1));
   dz=zz(yt(:,2))-zz(yt(:,1));
  
   uobs=[dx,dy,dz]./(sqrt(dx.^2+dy.^2+dz.^2)*ones(1,3)); % vecteur unitaire correspondant aux paires;
     
   % trouver les limites angulaires et identifier le cas
   for idir=1:ndir;  % boucle des directions
      
      da=uobs*u(idir,:)';
      id=abs(da)>=tol(idir);
      
      % ne conserver que les paires entrant dans l'angle recherch
      
      h=ht(id);
      y=yt(id,:);
      
      % Calcul des carres des ecarts des variables
      
      var=0.5*((v(y(:,1))-v(y(:,2))).^2);
      
      % Tri sur h et calcul des carres des ecarts correspondants
      
      for ic=1:ncl
         id=h>lclas(ic,1) & h<=lclas(ic,2);
         gexp(ic,2,idir)=gexp(ic,2,idir)+sum(id);
         gexp(ic,1,idir)=gexp(ic,1,idir)+sum(h(id));
         gexp(ic,3,idir)=gexp(ic,3,idir)+sum(var(id));                                         
      end
   end % boucle des directions
   
end % boucle des observations

% calculer le variogramme

for idir=1:ndir;
   id=gexp(:,2,idir)>0;
   gexp(id,1,idir)=gexp(id,1,idir)./gexp(id,2,idir);
   gexp(id,3,idir)=gexp(id,3,idir)./gexp(id,2,idir);
end

ymax=max(max(gexp(:,3,:)))*1.2;
xmax=max(max(gexp(:,1,:)))*1.2;
for idir=1:ndir   
   
   
   subplot(nplot,nplot,idir)
   plotgExp(gexp(:,1,idir),gexp(:,3,idir),gexp(:,2,idir),'+',6)
   axis([0 xmax, 0, ymax]);
   title(['Vario. exp. (dir, dip, reg) : (',num2str(90-vdir(idir,1),3),',',num2str(vdir(idir,2),3),',',num2str(vreg(idir),3),')']);
   xlabel('h')
   ylabel('\gamma(h)')
end

function x=poletocart(pole)
%
% fonction pour convertir des ples en coordonnes cartsiennes (systme main droite)
% pole: nx2 dir (azimut), pendage du pole angle positif vers le bas (retourne une coordonne z ngative)
deg2rad=pi/180;
pole=pole*deg2rad;

x(:,1)=cos(pole(:,2)).*sin(pole(:,1));
x(:,2)=cos(pole(:,2)).*cos(pole(:,1));
x(:,3)=-sin(pole(:,2));

